% dalsqgl - DAL with squared loss and grouped L1 regularization
%
% Overview:
%  Solves the optimization problem:
%   xx = argmin ||A*x-bb||^2 + lambda*||x||_G1
%  where
%   ||x||_G1 = sum(sqrt(sum(xx.^2)))
%  (grouped L1 norm)
%
% Syntax:
%  [xx,status]=dalsqgl(xx, A, bb, lambda, <opt>)
%
% Inputs:
%  xx     : initial solution ([ns,nc] or [nn,1] with opt.blks)
%  A      : the design matrix A ([mm,nn] or [mm,ns,nc])
%  bb     : the target vector ([mm,1])
%  lambda : the regularization constant
%  <opt>  : list of 'fieldname1', value1, 'filedname2', value2, ...
%   blks     : vector with elements corresponding to the size of
%              the blocks. If omitted, opt.blks = [ns, ns,..., ns],
%              where nc is the number of blocks.
%   stopcond : stopping condition, which can be
%              'pdg'  : Use relative primal dual gap (default)
%              'fval' : Use the objective function value
%           (see dal.m for other options)
% Outputs:
%  xx     : the final solution ([ns,nc] or [nn,1])
%  status : various status values
%
% Copyright(c) 2009 Ryota Tomioka
% This software is distributed under the MIT license. See license.txt

function [ww,status]=dalsqgl(ww, A, bb, lambda, varargin)

opt=propertylist2struct(varargin{:});
opt=set_defaults(opt,'solver','nt',...
                     'stopcond','pdg',...
                     'blks',[]);

if isempty(opt.blks)
  opt.blks=size(ww,1)*ones(1,size(ww,2));
  ww = ww(:);
end

prob.floss    = struct('p',@loss_sqp,'d',@loss_sqd,'args',{{bb}});
prob.fspec    = {@gl_spec, opt.blks};
prob.dnorm    = {@gl_dnorm,opt.blks};
prob.obj      = @objdalgl;
prob.softth   = @gl_softth;
prob.stopcond = ['stopcond_' opt.stopcond];
prob.ll       = -inf*ones(size(bb));
prob.uu       = inf*ones(size(bb));
prob.Ac       =[];
prob.bc       =[];
prob.info     = struct('blks',opt.blks);

if isequal(opt.solver,'cg')
  prob.hessMult = @hessMultdalgl;
end

if isequal(opt.stopcond,'fval')
  opt.feval = 1;
end

[mm,nn]=size(A);
prob.mm       = mm;
prob.nn       = nn;

[ww,uu,status]=dal(prob,ww,[],A,[],lambda,opt);

if all(opt.blks==opt.blks(1))
  ns=opt.blks(1);
  nc=length(ww)/ns;
  ww=reshape(ww, [ns,nc]);
end
